@@ -28,6 +28,6 @@ from pyomo.environ import *
 
 def define_components(mod):
     mod.administration_fees = Param(
-        mod.PERIODS,
-        initialize=lambda m, p: 1000000)
-    mod.Cost_Components_Per_Period.append('administration_fees')
+        mod.PERIODS, initialize=lambda m, p: 1000000, within=Any
+    )
+    mod.Cost_Components_Per_Period.append("administration_fees")
