@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2019 The Switch Authors. All rights reserved.
+# Copyright (c) 2015-2022 The Switch Authors. All rights reserved.
 # Licensed under the Apache License, Version 2.0, which is in the LICENSE file.
 
 """
@@ -11,10 +11,16 @@ from __future__ import division
 import os
 import csv
 from pyomo.environ import *
+from switch_model.utilities import unique_list
+
+dependencies = (
+    "switch_model.timescales",
+    "switch_model.balancing.load_zones",
+    "switch_model.energy_sources.properties.properties",
+    "switch_model.generators.core.build",
+    "switch_model.generators.core.dispatch",
+)
 
-dependencies = 'switch_model.timescales', 'switch_model.balancing.load_zones',\
-    'switch_model.energy_sources.properties.properties',\
-    'switch_model.generators.core.build', 'switch_model.generators.core.dispatch'
 
 def define_components(mod):
     """
@@ -42,7 +48,7 @@ def define_components(mod):
     ZONE_FUELS is the set of fuels available in load zones. It is specified
     as set of 2-member tuples of (load_zone, fuel).
 
-    zone_rfm[z, f] is the regional fuel market that supplies a a given load
+    zone_fuel_rfm[z, f] is the regional fuel market that supplies a a given load
     zone. Regional fuel markets may be referred to as fuel regions for
     brevity. A regional fuel market could be as small as a single load
     zone or as large as the entire study region. In general, each fuel
@@ -52,7 +58,7 @@ def define_components(mod):
     that define different regional markets.
 
     ZONE_RFMS is the set of all load-zone regional fuel market combinations.
-    It is the input data from which zone_rfm[z,f] is derived.
+    It is the input data from which zone_fuel_rfm[z,f] is derived.
 
     ZONES_IN_RFM[rfm] is an indexed set that lists the load zones
     within each regional fuel market.
@@ -204,51 +210,67 @@ def define_components(mod):
 
     """
 
-    mod.REGIONAL_FUEL_MARKETS = Set()
+    mod.REGIONAL_FUEL_MARKETS = Set(dimen=1)
     mod.rfm_fuel = Param(mod.REGIONAL_FUEL_MARKETS, within=mod.FUELS)
     mod.ZONE_RFMS = Set(
-        dimen=2, validate=lambda m, z, rfm: (
-            rfm in m.REGIONAL_FUEL_MARKETS and z in m.LOAD_ZONES))
+        dimen=2,
+        validate=lambda m, z, rfm: (
+            rfm in m.REGIONAL_FUEL_MARKETS and z in m.LOAD_ZONES
+        ),
+    )
     mod.ZONE_FUELS = Set(
-        dimen=2, initialize=lambda m: set(
-            (z, m.rfm_fuel[rfm]) for (z, rfm) in m.ZONE_RFMS))
+        dimen=2, initialize=lambda m: [(z, m.rfm_fuel[rfm]) for (z, rfm) in m.ZONE_RFMS]
+    )
 
-    def zone_rfm_init(m, load_zone, fuel):
+    def zone_fuel_rfm_init(m, load_zone, fuel):
+        # find first (only) matching rfm
         for (z, rfm) in m.ZONE_RFMS:
-            if(z == load_zone and fuel == m.rfm_fuel[rfm]):
+            if z == load_zone and fuel == m.rfm_fuel[rfm]:
                 return rfm
-    mod.zone_rfm = Param(
-        mod.ZONE_FUELS, within=mod.REGIONAL_FUEL_MARKETS,
-        initialize=zone_rfm_init)
-    mod.min_data_check('REGIONAL_FUEL_MARKETS', 'rfm_fuel', 'zone_rfm')
+
+    mod.zone_fuel_rfm = Param(
+        mod.ZONE_FUELS, within=mod.REGIONAL_FUEL_MARKETS, initialize=zone_fuel_rfm_init
+    )
+    mod.min_data_check("REGIONAL_FUEL_MARKETS", "rfm_fuel", "zone_fuel_rfm")
     mod.ZONES_IN_RFM = Set(
         mod.REGIONAL_FUEL_MARKETS,
-        initialize=lambda m, rfm: set(
-            z for (z, r) in m.ZONE_RFMS if r == rfm))
+        dimen=1,
+        initialize=lambda m, rfm: unique_list(z for (z, r) in m.ZONE_RFMS if r == rfm),
+    )
 
     # RFM_SUPPLY_TIERS = [(regional_fuel_market, period, supply_tier_index)...]
     mod.RFM_SUPPLY_TIERS = Set(
-        dimen=3, validate=lambda m, r, p, st: (
-            r in m.REGIONAL_FUEL_MARKETS and p in m.PERIODS))
-    mod.rfm_supply_tier_cost = Param(
-        mod.RFM_SUPPLY_TIERS, within=Reals)
+        dimen=3,
+        validate=lambda m, r, p, st: (r in m.REGIONAL_FUEL_MARKETS and p in m.PERIODS),
+    )
+    mod.rfm_supply_tier_cost = Param(mod.RFM_SUPPLY_TIERS, within=Reals)
     mod.rfm_supply_tier_limit = Param(
-        mod.RFM_SUPPLY_TIERS, within=NonNegativeReals, default=float('inf'))
+        mod.RFM_SUPPLY_TIERS, within=NonNegativeReals, default=float("inf")
+    )
     mod.min_data_check(
-        'RFM_SUPPLY_TIERS', 'rfm_supply_tier_cost', 'rfm_supply_tier_limit')
+        "RFM_SUPPLY_TIERS", "rfm_supply_tier_cost", "rfm_supply_tier_limit"
+    )
     mod.SUPPLY_TIERS_FOR_RFM_PERIOD = Set(
-        mod.REGIONAL_FUEL_MARKETS, mod.PERIODS, dimen=3,
-        initialize=lambda m, rfm, ip: set(
-            (r, p, st) for (r, p, st) in m.RFM_SUPPLY_TIERS
-            if r == rfm and p == ip))
+        mod.REGIONAL_FUEL_MARKETS,
+        mod.PERIODS,
+        dimen=3,
+        initialize=lambda m, rfm, ip: [
+            (r, p, st) for (r, p, st) in m.RFM_SUPPLY_TIERS if r == rfm and p == ip
+        ],
+    )
 
     mod.ConsumeFuelTier = Var(
         mod.RFM_SUPPLY_TIERS,
         domain=NonNegativeReals,
         bounds=lambda m, rfm, p, st: (
-            0, (m.rfm_supply_tier_limit[rfm, p, st]
-                if value(m.rfm_supply_tier_limit[rfm, p, st]) != float('inf')
-                else None)))
+            0,
+            (
+                m.rfm_supply_tier_limit[rfm, p, st]
+                if value(m.rfm_supply_tier_limit[rfm, p, st]) != float("inf")
+                else None
+            ),
+        ),
+    )
     # The if statement in the upper bound of ConsumeFuelTier is a
     # work-around for a Pyomo bug in writing a cpxlp problem file for
     # glpk. Lines 771-774 of pyomo/repn/plugins/cpxlp.py prints '<= inf'
@@ -259,90 +281,127 @@ def define_components(mod):
     # solvers is: 0, m.rfm_supply_tier_limit[rfm, p, st]))
 
     mod.FuelConsumptionInMarket = Expression(
-        mod.REGIONAL_FUEL_MARKETS, mod.PERIODS,
+        mod.REGIONAL_FUEL_MARKETS,
+        mod.PERIODS,
         rule=lambda m, rfm, p: sum(
             m.ConsumeFuelTier[rfm_supply_tier]
-                for rfm_supply_tier in m.SUPPLY_TIERS_FOR_RFM_PERIOD[rfm, p]))
+            for rfm_supply_tier in m.SUPPLY_TIERS_FOR_RFM_PERIOD[rfm, p]
+        ),
+    )
 
     # Ensure that adjusted fuel costs of unbounded supply tiers are not
     # negative because that would create an unbounded optimization
     # problem.
     def zone_fuel_cost_adder_validate(model, val, z, fuel, p):
-        rfm = model.zone_rfm[z, fuel]
+        rfm = model.zone_fuel_rfm[z, fuel]
         for rfm_supply_tier in model.SUPPLY_TIERS_FOR_RFM_PERIOD[rfm, p]:
-            if(val + model.rfm_supply_tier_cost[rfm_supply_tier] < 0 and
-               model.rfm_supply_tier_limit[rfm_supply_tier] == float('inf')):
+            if val + model.rfm_supply_tier_cost[
+                rfm_supply_tier
+            ] < 0 and model.rfm_supply_tier_limit[rfm_supply_tier] == float("inf"):
                 return False
         return True
+
     mod.zone_fuel_cost_adder = Param(
-        mod.ZONE_FUELS, mod.PERIODS,
-        within=Reals, default=0, validate=zone_fuel_cost_adder_validate)
+        mod.ZONE_FUELS,
+        mod.PERIODS,
+        within=Reals,
+        default=0,
+        validate=zone_fuel_cost_adder_validate,
+    )
 
     # Summarize annual fuel costs for the objective function
     def rfm_annual_costs(m, rfm, p):
         return sum(
             m.ConsumeFuelTier[rfm_st] * m.rfm_supply_tier_cost[rfm_st]
-            for rfm_st in m.SUPPLY_TIERS_FOR_RFM_PERIOD[rfm, p])
+            for rfm_st in m.SUPPLY_TIERS_FOR_RFM_PERIOD[rfm, p]
+        )
+
     mod.FuelCostsPerPeriod = Expression(
         mod.PERIODS,
         rule=lambda m, p: sum(
-            rfm_annual_costs(m, rfm, p)
-            for rfm in m.REGIONAL_FUEL_MARKETS))
-    mod.Cost_Components_Per_Period.append('FuelCostsPerPeriod')
+            rfm_annual_costs(m, rfm, p) for rfm in m.REGIONAL_FUEL_MARKETS
+        ),
+    )
+    mod.Cost_Components_Per_Period.append("FuelCostsPerPeriod")
 
     # Components to link aggregate fuel consumption from project
     # dispatch into market framework
 
     def GENS_FOR_RFM_PERIOD_rule(m, rfm, p):
-        # Construct and cache a set of gens for each zone/fuel/period, then
-        # return lists of gens for each rfm/period as needed
+        # Construct and cache list of gens for each rfm/period, then return them
+        # as needed
         try:
             d = m.GENS_FOR_RFM_PERIOD_dict
         except AttributeError:
             d = m.GENS_FOR_RFM_PERIOD_dict = dict()
-            # d uses (zone, fuel, period) as key; could use (rfm, period) as key
-            # if m.zone_fuel_rfm (back-lookup) existed
             for g in m.FUEL_BASED_GENS:
                 for f in m.FUELS_FOR_GEN[g]:
-                    for p_ in m.PERIODS_FOR_GEN[g]:
-                        d.setdefault((m.gen_load_zone[g], f, p_), []).append(g)
-        relevant_gens = [
-            g
-            for z in m.ZONES_IN_RFM[rfm]
-            for g in d.pop((z, m.rfm_fuel[rfm], p), []) # pop releases memory
-        ]
-        return relevant_gens
+                    try:
+                        _rfm = m.zone_fuel_rfm[m.gen_load_zone[g], f]
+                    except KeyError:  # no rfm provides this fuel
+                        pass
+                    else:
+                        for _p in m.PERIODS_FOR_GEN[g]:
+                            d.setdefault((_rfm, _p), []).append(g)
+        return d.pop((rfm, p), [])  # pop releases memory
+
     mod.GENS_FOR_RFM_PERIOD = Set(
-        mod.REGIONAL_FUEL_MARKETS, mod.PERIODS,
-        initialize=GENS_FOR_RFM_PERIOD_rule
+        mod.REGIONAL_FUEL_MARKETS,
+        mod.PERIODS,
+        dimen=1,
+        initialize=GENS_FOR_RFM_PERIOD_rule,
     )
+
     def Enforce_Fuel_Consumption_rule(m, rfm, p):
         return m.FuelConsumptionInMarket[rfm, p] == sum(
             m.GenFuelUseRate[g, t, m.rfm_fuel[rfm]] * m.tp_weight_in_year[t]
             for g in m.GENS_FOR_RFM_PERIOD[rfm, p]
             for t in m.TPS_IN_PERIOD[p]
-    )
+        )
+
     mod.Enforce_Fuel_Consumption = Constraint(
-        mod.REGIONAL_FUEL_MARKETS, mod.PERIODS,
-        rule=Enforce_Fuel_Consumption_rule)
+        mod.REGIONAL_FUEL_MARKETS, mod.PERIODS, rule=Enforce_Fuel_Consumption_rule
+    )
 
     mod.GEN_TP_FUELS_UNAVAILABLE = Set(
+        dimen=3,
         initialize=mod.GEN_TP_FUELS,
-        filter=lambda m, g, t, f: \
-            (m.gen_load_zone[g], f) not in m.ZONE_FUELS)
+        filter=lambda m, g, t, f: (m.gen_load_zone[g], f) not in m.ZONE_FUELS,
+    )
     mod.Enforce_Fuel_Unavailability = Constraint(
         mod.GEN_TP_FUELS_UNAVAILABLE,
-        rule=lambda m, g, t, f: m.GenFuelUseRate[g, t, f] == 0)
+        rule=lambda m, g, t, f: m.GenFuelUseRate[g, t, f] == 0,
+    )
 
-
-    # Calculate average fuel costs to allow post-optimization inspection
-    # and cost allocation.
     mod.AverageFuelCosts = Expression(
-        mod.REGIONAL_FUEL_MARKETS, mod.PERIODS,
+        mod.REGIONAL_FUEL_MARKETS,
+        mod.PERIODS,
+        doc="Average fuel costs to allow post-optimization inspection "
+        "and cost allocation.",
         rule=lambda m, rfm, p: (
-            rfm_annual_costs(m, rfm, p) /
-            sum(m.ConsumeFuelTier[rfm_st]
-                for rfm_st in m.SUPPLY_TIERS_FOR_RFM_PERIOD[rfm, p])))
+            rfm_annual_costs(m, rfm, p)
+            / (
+                # Avoid divide-by-zero errors if no fuel is consumed.
+                m.FuelConsumptionInMarket[rfm, p]
+                + 0.0001
+            )
+        ),
+    )
+
+    def GenFuelCosts_rule(m, g, t, f):
+        try:
+            rfm = m.zone_fuel_rfm[m.gen_load_zone[g], f]
+        except KeyError:  # Fuel is unavailable
+            return 0.0
+        p = m.tp_period[t]
+        return m.GenFuelUseRate[g, t, f] * m.AverageFuelCosts[rfm, p]
+
+    mod.GenFuelCosts = Expression(
+        mod.GEN_TP_FUELS,
+        doc="Average cost of fuel consumption, $/hr for post-optimization "
+        "reporting.",
+        rule=GenFuelCosts_rule,
+    )
 
 
 def load_inputs(mod, switch_data, inputs_dir):
@@ -383,91 +442,120 @@ def load_inputs(mod, switch_data, inputs_dir):
     # message if some columns are not found.
 
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'regional_fuel_markets.csv'),
-        select=('regional_fuel_market', 'fuel'),
+        filename=os.path.join(inputs_dir, "regional_fuel_markets.csv"),
+        select=("regional_fuel_market", "fuel"),
         index=mod.REGIONAL_FUEL_MARKETS,
-        param=(mod.rfm_fuel))
+        param=(mod.rfm_fuel),
+    )
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'fuel_supply_curves.csv'),
-        select=('regional_fuel_market', 'period', 'tier', 'unit_cost',
-                'max_avail_at_cost'),
+        filename=os.path.join(inputs_dir, "fuel_supply_curves.csv"),
+        select=(
+            "regional_fuel_market",
+            "period",
+            "tier",
+            "unit_cost",
+            "max_avail_at_cost",
+        ),
         index=mod.RFM_SUPPLY_TIERS,
-        param=(mod.rfm_supply_tier_cost, mod.rfm_supply_tier_limit))
+        param=(mod.rfm_supply_tier_cost, mod.rfm_supply_tier_limit),
+    )
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'zone_to_regional_fuel_market.csv'),
-        set=mod.ZONE_RFMS)
+        filename=os.path.join(inputs_dir, "zone_to_regional_fuel_market.csv"),
+        set=mod.ZONE_RFMS,
+    )
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'zone_fuel_cost_diff.csv'),
+        filename=os.path.join(inputs_dir, "zone_fuel_cost_diff.csv"),
         optional=True,
-        select=('load_zone', 'fuel', 'period', 'fuel_cost_adder'),
-        param=(mod.zone_fuel_cost_adder))
+        select=("load_zone", "fuel", "period", "fuel_cost_adder"),
+        param=(mod.zone_fuel_cost_adder),
+    )
 
     # Load a simple specifications of costs if the file exists. The
     # actual loading, error checking, and casting into a supply curve is
     # slightly complicated, so I moved that logic to a separate function.
-    path = os.path.join(inputs_dir, 'fuel_cost.csv')
+    path = os.path.join(inputs_dir, "fuel_cost.csv")
     if os.path.isfile(path):
         _load_simple_cost_data(mod, switch_data, path)
 
 
 def _load_simple_cost_data(mod, switch_data, path):
-    with open(path, 'r') as simple_cost_file:
-        simple_cost_dat = list(csv.DictReader(simple_cost_file, delimiter=','))
+    with open(path, "r") as simple_cost_file:
+        simple_cost_dat = list(csv.DictReader(simple_cost_file, delimiter=","))
         # Scan once for error checking
         for row in simple_cost_dat:
-            z = row['load_zone']
-            f = row['fuel']
-            p = int(row['period'])
-            f_cost = float(row['fuel_cost'])
+            z = row["load_zone"]
+            f = row["fuel"]
+            p = int(row["period"])
+            f_cost = float(row["fuel_cost"])
             # Basic data validity checks
-            if z not in switch_data.data(name='LOAD_ZONES'):
+            if z not in switch_data.data(name="LOAD_ZONES"):
                 raise ValueError(
-                    "Load zone " + z + " in zone_simple_fuel_cost.csv is not " +
-                    "a known load zone from load_zones.csv.")
-            if f not in switch_data.data(name='FUELS'):
+                    "Load zone "
+                    + z
+                    + " in zone_simple_fuel_cost.csv is not "
+                    + "a known load zone from load_zones.csv."
+                )
+            if f not in switch_data.data(name="FUELS"):
                 raise ValueError(
-                    "Fuel " + f + " in zone_simple_fuel_cost.csv is not " +
-                    "a known fuel from fuels.csv.")
-            if p not in switch_data.data(name='PERIODS'):
+                    "Fuel "
+                    + f
+                    + " in zone_simple_fuel_cost.csv is not "
+                    + "a known fuel from fuels.csv."
+                )
+            if p not in switch_data.data(name="PERIODS"):
                 raise ValueError(
-                    "Period " + p + " in zone_simple_fuel_cost.csv is not " +
-                    "a known investment period.")
+                    "Period "
+                    + p
+                    + " in zone_simple_fuel_cost.csv is not "
+                    + "a known investment period."
+                )
             # Make sure they aren't overriding a supply curve or
             # regional fuel market defined in previous files.
-            for (z, rfm) in switch_data.data(name='ZONE_RFMS'):
-                if(z == z and
-                   switch_data.data(name='rfm_fuel')[rfm] == f):
+            for (z, rfm) in switch_data.data(name="ZONE_RFMS"):
+                if z == z and switch_data.data(name="rfm_fuel")[rfm] == f:
                     raise ValueError(
-                        "The supply for fuel '" + f + "' for load_zone '" + z +
-                        "' was already registered with the regional fuel " +
-                        "market '" + rfm + "', so you cannot " +
-                        "specify a simple fuel cost for it in " +
-                        "zone_simple_fuel_cost.csv. You either need to delete " +
-                        "that entry from zone_to_regional_fuel_market.csv, or " +
-                        "remove those entries in zone_simple_fuel_cost.csv.")
+                        "The supply for fuel '"
+                        + f
+                        + "' for load_zone '"
+                        + z
+                        + "' was already registered with the regional fuel "
+                        + "market '"
+                        + rfm
+                        + "', so you cannot "
+                        + "specify a simple fuel cost for it in "
+                        + "zone_simple_fuel_cost.csv. You either need to delete "
+                        + "that entry from zone_to_regional_fuel_market.csv, or "
+                        + "remove those entries in zone_simple_fuel_cost.csv."
+                    )
             # Make a new single-load zone regional fuel market.
             rfm = z + "_" + f
-            if rfm in switch_data.data(name='REGIONAL_FUEL_MARKETS'):
+            if rfm in switch_data.data(name="REGIONAL_FUEL_MARKETS"):
                 raise ValueError(
-                    "Trying to construct a simple Regional Fuel Market " +
-                    "called " + rfm + " from data in zone_simple_fuel_cost.csv" +
-                    ", but an RFM of that name already exists. Bailing out!")
+                    "Trying to construct a simple Regional Fuel Market "
+                    + "called "
+                    + rfm
+                    + " from data in zone_simple_fuel_cost.csv"
+                    + ", but an RFM of that name already exists. Bailing out!"
+                )
         # Scan again and actually import the data
         for row in simple_cost_dat:
-            z = row['load_zone']
-            f = row['fuel']
-            p = int(row['period'])
-            f_cost = float(row['fuel_cost'])
+            z = row["load_zone"]
+            f = row["fuel"]
+            p = int(row["period"])
+            f_cost = float(row["fuel_cost"])
             # Make a new single-load zone regional fuel market unless we
             # already defined one in this loop for a different period.
             rfm = z + "_" + f
-            if(rfm not in switch_data.data(name='REGIONAL_FUEL_MARKETS')):
-                switch_data.data(name='REGIONAL_FUEL_MARKETS').append(rfm)
-                switch_data.data(name='rfm_fuel')[rfm] = f
-                switch_data.data(name='ZONE_RFMS').append((z, rfm))
+            if rfm not in switch_data.data(name="REGIONAL_FUEL_MARKETS"):
+                switch_data.data(name="REGIONAL_FUEL_MARKETS").append(rfm)
+                switch_data.data(name="rfm_fuel")[rfm] = f
+                switch_data.data(name="ZONE_RFMS").append((z, rfm))
             # Make a single supply tier for this RFM and period
             st = 0
-            switch_data.data(name='RFM_SUPPLY_TIERS').append((rfm, p, st))
-            switch_data.data(name='rfm_supply_tier_cost')[rfm, p, st] = f_cost
-            switch_data.data(name='rfm_supply_tier_limit')[rfm, p, st] = \
-                float('inf')
+            switch_data.data(name="RFM_SUPPLY_TIERS").append((rfm, p, st))
+            switch_data.data(name="rfm_supply_tier_cost")[rfm, p, st] = f_cost
+            # No need to specify an upper limit, since default is infinity
+            # (and this creates inf values in the data portal that could get
+            # written out to .dat files for PySP, which would be unable to read
+            # those values in Pyomo 5.7+)
+            # switch_data.data(name="rfm_supply_tier_limit")[rfm, p, st] = float("inf")
