@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2019 The Switch Authors. All rights reserved.
+# Copyright (c) 2015-2022 The Switch Authors. All rights reserved.
 # Licensed under the Apache License, Version 2.0, which is in the LICENSE file.
 """
 Defines generation projects build-outs.
@@ -9,9 +9,15 @@ import os
 from pyomo.environ import *
 from switch_model.financials import capital_recovery_factor as crf
 from switch_model.reporting import write_table
+from switch_model.utilities import unique_list
+
+dependencies = (
+    "switch_model.timescales",
+    "switch_model.balancing.load_zones",
+    "switch_model.financials",
+    "switch_model.energy_sources.properties.properties",
+)
 
-dependencies = 'switch_model.timescales', 'switch_model.balancing.load_zones',\
-    'switch_model.financials', 'switch_model.energy_sources.properties.properties'
 
 def define_components(mod):
     """
@@ -87,7 +93,7 @@ def define_components(mod):
     only includes existing or planned projects that are not subject to
     optimization.
 
-    gen_predetermined_cap[(g, build_year) in PREDETERMINED_GEN_BLD_YRS] is
+    build_gen_predetermined[(g, build_year) in PREDETERMINED_GEN_BLD_YRS] is
     a parameter that describes how much capacity was built in the past
     for existing projects, or is planned to be built for future projects.
 
@@ -117,7 +123,7 @@ def define_components(mod):
     force project build-outs to meet the minimum build requirements for
     generation technologies that have those requirements. They force BuildGen
     to be 0 when BuildMinGenCap is 0, and to be greater than
-    g_min_build_capacity when BuildMinGenCap is 1. In the latter case,
+    gen_min_build_capacity when BuildMinGenCap is 1. In the latter case,
     the upper constraint should be non-binding; the upper limit is set to 10
     times the peak non-conincident demand of the entire system.
 
@@ -132,6 +138,9 @@ def define_components(mod):
     for the given project in the given period. For some project-period
     combinations, this will be an empty set.
 
+    PERIODS_FOR_GEN[g] is the set of all periods when generation project
+    g could potentially be operated.
+
     GEN_PERIODS describes periods in which generation projects
     could be operational. Unlike the related sets above, it is not
     indexed. Instead it is specified as a set of (g, period)
@@ -184,33 +193,48 @@ def define_components(mod):
     - Allow early capacity retirements with savings on fixed O&M
 
     """
-    mod.GENERATION_PROJECTS = Set()
-    mod.gen_dbid = Param(mod.GENERATION_PROJECTS, default=lambda m, g: g)
-    mod.gen_tech = Param(mod.GENERATION_PROJECTS)
-    mod.GENERATION_TECHNOLOGIES = Set(initialize=lambda m:
-        {m.gen_tech[g] for g in m.GENERATION_PROJECTS}
+    mod.GENERATION_PROJECTS = Set(dimen=1)
+    mod.gen_dbid = Param(mod.GENERATION_PROJECTS, default=lambda m, g: g, within=Any)
+    mod.gen_tech = Param(mod.GENERATION_PROJECTS, within=Any)
+    mod.GENERATION_TECHNOLOGIES = Set(
+        dimen=1,
+        initialize=lambda m: unique_list(m.gen_tech[g] for g in m.GENERATION_PROJECTS),
+    )
+    mod.gen_energy_source = Param(
+        mod.GENERATION_PROJECTS,
+        validate=lambda m, val, g: val in m.ENERGY_SOURCES or val == "multiple",
+        within=Any,
     )
-    mod.gen_energy_source = Param(mod.GENERATION_PROJECTS,
-        validate=lambda m,val,g: val in m.ENERGY_SOURCES or val == "multiple")
     mod.gen_load_zone = Param(mod.GENERATION_PROJECTS, within=mod.LOAD_ZONES)
     mod.gen_max_age = Param(mod.GENERATION_PROJECTS, within=PositiveIntegers)
     mod.gen_is_variable = Param(mod.GENERATION_PROJECTS, within=Boolean)
     mod.gen_is_baseload = Param(mod.GENERATION_PROJECTS, within=Boolean, default=False)
     mod.gen_is_cogen = Param(mod.GENERATION_PROJECTS, within=Boolean, default=False)
-    mod.gen_is_distributed = Param(mod.GENERATION_PROJECTS, within=Boolean, default=False)
-    mod.gen_scheduled_outage_rate = Param(mod.GENERATION_PROJECTS,
-        within=PercentFraction, default=0)
-    mod.gen_forced_outage_rate = Param(mod.GENERATION_PROJECTS,
-        within=PercentFraction, default=0)
-    mod.min_data_check('GENERATION_PROJECTS', 'gen_tech', 'gen_energy_source',
-        'gen_load_zone', 'gen_max_age', 'gen_is_variable')
+    mod.gen_is_distributed = Param(
+        mod.GENERATION_PROJECTS, within=Boolean, default=False
+    )
+    mod.gen_scheduled_outage_rate = Param(
+        mod.GENERATION_PROJECTS, within=PercentFraction, default=0
+    )
+    mod.gen_forced_outage_rate = Param(
+        mod.GENERATION_PROJECTS, within=PercentFraction, default=0
+    )
+    mod.min_data_check(
+        "GENERATION_PROJECTS",
+        "gen_tech",
+        "gen_energy_source",
+        "gen_load_zone",
+        "gen_max_age",
+        "gen_is_variable",
+    )
 
     """Construct GENS_* indexed sets efficiently with a
     'construction dictionary' pattern: on the first call, make a single
     traversal through all generation projects to generate a complete index,
     use that for subsequent lookups, and clean up at the last call."""
+
     def GENS_IN_ZONE_init(m, z):
-        if not hasattr(m, 'GENS_IN_ZONE_dict'):
+        if not hasattr(m, "GENS_IN_ZONE_dict"):
             m.GENS_IN_ZONE_dict = {_z: [] for _z in m.LOAD_ZONES}
             for g in m.GENERATION_PROJECTS:
                 m.GENS_IN_ZONE_dict[m.gen_load_zone[g]].append(g)
@@ -218,22 +242,26 @@ def define_components(mod):
         if not m.GENS_IN_ZONE_dict:
             del m.GENS_IN_ZONE_dict
         return result
-    mod.GENS_IN_ZONE = Set(
-        mod.LOAD_ZONES,
-        initialize=GENS_IN_ZONE_init
-    )
+
+    mod.GENS_IN_ZONE = Set(mod.LOAD_ZONES, dimen=1, initialize=GENS_IN_ZONE_init)
     mod.VARIABLE_GENS = Set(
+        dimen=1,
         initialize=mod.GENERATION_PROJECTS,
-        filter=lambda m, g: m.gen_is_variable[g])
+        filter=lambda m, g: m.gen_is_variable[g],
+    )
     mod.VARIABLE_GENS_IN_ZONE = Set(
         mod.LOAD_ZONES,
-        initialize=lambda m, z: [g for g in m.GENS_IN_ZONE[z] if m.gen_is_variable[g]])
+        dimen=1,
+        initialize=lambda m, z: [g for g in m.GENS_IN_ZONE[z] if m.gen_is_variable[g]],
+    )
     mod.BASELOAD_GENS = Set(
+        dimen=1,
         initialize=mod.GENERATION_PROJECTS,
-        filter=lambda m, g: m.gen_is_baseload[g])
+        filter=lambda m, g: m.gen_is_baseload[g],
+    )
 
     def GENS_BY_TECHNOLOGY_init(m, t):
-        if not hasattr(m, 'GENS_BY_TECH_dict'):
+        if not hasattr(m, "GENS_BY_TECH_dict"):
             m.GENS_BY_TECH_dict = {_t: [] for _t in m.GENERATION_TECHNOLOGIES}
             for g in m.GENERATION_PROJECTS:
                 m.GENS_BY_TECH_dict[m.gen_tech[g]].append(g)
@@ -241,50 +269,80 @@ def define_components(mod):
         if not m.GENS_BY_TECH_dict:
             del m.GENS_BY_TECH_dict
         return result
+
     mod.GENS_BY_TECHNOLOGY = Set(
-        mod.GENERATION_TECHNOLOGIES,
-        initialize=GENS_BY_TECHNOLOGY_init
+        mod.GENERATION_TECHNOLOGIES, dimen=1, initialize=GENS_BY_TECHNOLOGY_init
     )
 
-    mod.CAPACITY_LIMITED_GENS = Set(within=mod.GENERATION_PROJECTS)
+    mod.CAPACITY_LIMITED_GENS = Set(within=mod.GENERATION_PROJECTS, dimen=1)
     mod.gen_capacity_limit_mw = Param(
-        mod.CAPACITY_LIMITED_GENS, within=NonNegativeReals)
-    mod.DISCRETELY_SIZED_GENS = Set(within=mod.GENERATION_PROJECTS)
-    mod.gen_unit_size = Param(
-        mod.DISCRETELY_SIZED_GENS, within=PositiveReals)
-    mod.CCS_EQUIPPED_GENS = Set(within=mod.GENERATION_PROJECTS)
+        mod.CAPACITY_LIMITED_GENS, within=NonNegativeReals
+    )
+    mod.DISCRETELY_SIZED_GENS = Set(dimen=1, within=mod.GENERATION_PROJECTS)
+    mod.gen_unit_size = Param(mod.DISCRETELY_SIZED_GENS, within=PositiveReals)
+    mod.CCS_EQUIPPED_GENS = Set(dimen=1, within=mod.GENERATION_PROJECTS)
     mod.gen_ccs_capture_efficiency = Param(
-        mod.CCS_EQUIPPED_GENS, within=PercentFraction)
-    mod.gen_ccs_energy_load = Param(
-        mod.CCS_EQUIPPED_GENS, within=PercentFraction)
+        mod.CCS_EQUIPPED_GENS, within=PercentFraction
+    )
+    mod.gen_ccs_energy_load = Param(mod.CCS_EQUIPPED_GENS, within=PercentFraction)
 
     mod.gen_uses_fuel = Param(
         mod.GENERATION_PROJECTS,
+        within=Boolean,
         initialize=lambda m, g: (
-            m.gen_energy_source[g] in m.FUELS
-                or m.gen_energy_source[g] == "multiple"))
+            m.gen_energy_source[g] in m.FUELS or m.gen_energy_source[g] == "multiple"
+        ),
+    )
     mod.NON_FUEL_BASED_GENS = Set(
+        dimen=1,
         initialize=mod.GENERATION_PROJECTS,
-        filter=lambda m, g: not m.gen_uses_fuel[g])
+        filter=lambda m, g: not m.gen_uses_fuel[g],
+    )
     mod.FUEL_BASED_GENS = Set(
+        dimen=1,
         initialize=mod.GENERATION_PROJECTS,
-        filter=lambda m, g: m.gen_uses_fuel[g])
+        filter=lambda m, g: m.gen_uses_fuel[g],
+    )
 
-    mod.gen_full_load_heat_rate = Param(
-        mod.FUEL_BASED_GENS,
-        within=NonNegativeReals)
+    mod.gen_full_load_heat_rate = Param(mod.FUEL_BASED_GENS, within=NonNegativeReals)
     mod.MULTIFUEL_GENS = Set(
+        dimen=1,
+        within=Any,
         initialize=mod.GENERATION_PROJECTS,
-        filter=lambda m, g: m.gen_energy_source[g] == "multiple")
-    mod.FUELS_FOR_MULTIFUEL_GEN = Set(mod.MULTIFUEL_GENS, within=mod.FUELS)
-    mod.FUELS_FOR_GEN = Set(mod.FUEL_BASED_GENS,
+        filter=lambda m, g: m.gen_energy_source[g] == "multiple",
+    )
+    mod.MULTI_FUEL_GEN_FUELS = Set(
+        dimen=2, validate=lambda m, g, f: g in m.MULTIFUEL_GENS and f in m.FUELS
+    )
+
+    def FUELS_FOR_MULTIFUEL_GEN_init(m, g):
+        if not hasattr(m, "FUELS_FOR_MULTIFUEL_GEN_dict"):
+            m.FUELS_FOR_MULTIFUEL_GEN_dict = {_g: [] for _g in m.MULTIFUEL_GENS}
+            for _g, f in m.MULTI_FUEL_GEN_FUELS:
+                m.FUELS_FOR_MULTIFUEL_GEN_dict[_g].append(f)
+        result = m.FUELS_FOR_MULTIFUEL_GEN_dict.pop(g)
+        if not m.FUELS_FOR_MULTIFUEL_GEN_dict:
+            del m.FUELS_FOR_MULTIFUEL_GEN_dict
+        return result
+
+    mod.FUELS_FOR_MULTIFUEL_GEN = Set(
+        mod.MULTIFUEL_GENS,
+        dimen=1,
+        within=mod.FUELS,
+        initialize=FUELS_FOR_MULTIFUEL_GEN_init,
+    )
+    mod.FUELS_FOR_GEN = Set(
+        mod.FUEL_BASED_GENS,
+        dimen=1,
         initialize=lambda m, g: (
             m.FUELS_FOR_MULTIFUEL_GEN[g]
             if g in m.MULTIFUEL_GENS
-            else [m.gen_energy_source[g]]))
+            else [m.gen_energy_source[g]]
+        ),
+    )
 
     def GENS_BY_ENERGY_SOURCE_init(m, e):
-        if not hasattr(m, 'GENS_BY_ENERGY_dict'):
+        if not hasattr(m, "GENS_BY_ENERGY_dict"):
             m.GENS_BY_ENERGY_dict = {_e: [] for _e in m.ENERGY_SOURCES}
             for g in m.GENERATION_PROJECTS:
                 if g in m.FUEL_BASED_GENS:
@@ -296,34 +354,34 @@ def define_components(mod):
         if not m.GENS_BY_ENERGY_dict:
             del m.GENS_BY_ENERGY_dict
         return result
+
     mod.GENS_BY_ENERGY_SOURCE = Set(
-        mod.ENERGY_SOURCES,
-        initialize=GENS_BY_ENERGY_SOURCE_init
+        mod.ENERGY_SOURCES, dimen=1, initialize=GENS_BY_ENERGY_SOURCE_init
     )
     mod.GENS_BY_NON_FUEL_ENERGY_SOURCE = Set(
         mod.NON_FUEL_ENERGY_SOURCES,
-        initialize=lambda m, s: m.GENS_BY_ENERGY_SOURCE[s]
+        dimen=1,
+        initialize=lambda m, s: m.GENS_BY_ENERGY_SOURCE[s],
     )
     mod.GENS_BY_FUEL = Set(
-        mod.FUELS,
-        initialize=lambda m, f: m.GENS_BY_ENERGY_SOURCE[f]
+        mod.FUELS, dimen=1, initialize=lambda m, f: m.GENS_BY_ENERGY_SOURCE[f]
     )
 
-    mod.PREDETERMINED_GEN_BLD_YRS = Set(
-        dimen=2)
+    mod.PREDETERMINED_GEN_BLD_YRS = Set(dimen=2)
     mod.GEN_BLD_YRS = Set(
         dimen=2,
         validate=lambda m, g, bld_yr: (
-            (g, bld_yr) in m.PREDETERMINED_GEN_BLD_YRS or
-            (g, bld_yr) in m.GENERATION_PROJECTS * m.PERIODS))
+            (g, bld_yr) in m.PREDETERMINED_GEN_BLD_YRS
+            or (g, bld_yr) in m.GENERATION_PROJECTS * m.PERIODS
+        ),
+    )
     mod.NEW_GEN_BLD_YRS = Set(
-        dimen=2,
-        initialize=lambda m: m.GEN_BLD_YRS - m.PREDETERMINED_GEN_BLD_YRS)
-    mod.gen_predetermined_cap = Param(
-        mod.PREDETERMINED_GEN_BLD_YRS,
-        within=NonNegativeReals)
-    mod.min_data_check('gen_predetermined_cap')
-
+        dimen=2, initialize=lambda m: m.GEN_BLD_YRS - m.PREDETERMINED_GEN_BLD_YRS
+    )
+    mod.build_gen_predetermined = Param(
+        mod.PREDETERMINED_GEN_BLD_YRS, within=NonNegativeReals
+    )
+    mod.min_data_check("build_gen_predetermined")
 
     def gen_build_can_operate_in_period(m, g, build_year, period):
         if build_year in m.PERIODS:
@@ -331,9 +389,7 @@ def define_components(mod):
         else:
             online = build_year
         retirement = online + m.gen_max_age[g]
-        return (
-            online <= m.period_start[period] < retirement
-        )
+        return online <= m.period_start[period] < retirement
         # This is probably more correct, but is a different behavior
         # mid_period = m.period_start[period] + 0.5 * m.period_length_years[period]
         # return online <= m.period_start[period] and mid_period <= retirement
@@ -341,39 +397,50 @@ def define_components(mod):
     # The set of periods when a project built in a certain year will be online
     mod.PERIODS_FOR_GEN_BLD_YR = Set(
         mod.GEN_BLD_YRS,
+        dimen=1,
         within=mod.PERIODS,
         ordered=True,
-        initialize=lambda m, g, bld_yr: set(
-            period for period in m.PERIODS
-            if gen_build_can_operate_in_period(m, g, bld_yr, period)))
+        initialize=lambda m, g, bld_yr: [
+            period
+            for period in m.PERIODS
+            if gen_build_can_operate_in_period(m, g, bld_yr, period)
+        ],
+    )
     # The set of build years that could be online in the given period
     # for the given project.
     mod.BLD_YRS_FOR_GEN_PERIOD = Set(
-        mod.GENERATION_PROJECTS, mod.PERIODS,
-        initialize=lambda m, g, period: set(
-            bld_yr for (gen, bld_yr) in m.GEN_BLD_YRS
-            if gen == g and
-               gen_build_can_operate_in_period(m, g, bld_yr, period)))
+        mod.GENERATION_PROJECTS,
+        mod.PERIODS,
+        dimen=1,
+        initialize=lambda m, g, period: unique_list(
+            bld_yr
+            for (gen, bld_yr) in m.GEN_BLD_YRS
+            if gen == g and gen_build_can_operate_in_period(m, g, bld_yr, period)
+        ),
+    )
     # The set of periods when a generator is available to run
     mod.PERIODS_FOR_GEN = Set(
         mod.GENERATION_PROJECTS,
-        initialize=lambda m, g: [p for p in m.PERIODS if len(m.BLD_YRS_FOR_GEN_PERIOD[g, p]) > 0]
+        dimen=1,
+        initialize=lambda m, g: [
+            p for p in m.PERIODS if len(m.BLD_YRS_FOR_GEN_PERIOD[g, p]) > 0
+        ],
     )
 
     def bounds_BuildGen(model, g, bld_yr):
-        if((g, bld_yr) in model.PREDETERMINED_GEN_BLD_YRS):
-            return (model.gen_predetermined_cap[g, bld_yr],
-                    model.gen_predetermined_cap[g, bld_yr])
-        elif(g in model.CAPACITY_LIMITED_GENS):
+        if (g, bld_yr) in model.PREDETERMINED_GEN_BLD_YRS:
+            return (
+                model.build_gen_predetermined[g, bld_yr],
+                model.build_gen_predetermined[g, bld_yr],
+            )
+        elif g in model.CAPACITY_LIMITED_GENS:
             # This does not replace Max_Build_Potential because
             # Max_Build_Potential applies across all build years.
             return (0, model.gen_capacity_limit_mw[g])
         else:
             return (0, None)
-    mod.BuildGen = Var(
-        mod.GEN_BLD_YRS,
-        within=NonNegativeReals,
-        bounds=bounds_BuildGen)
+
+    mod.BuildGen = Var(mod.GEN_BLD_YRS, within=NonNegativeReals, bounds=bounds_BuildGen)
     # Some projects are retired before the first study period, so they
     # don't appear in the objective function or any constraints.
     # In this case, pyomo may leave the variable value undefined even
@@ -383,10 +450,11 @@ def define_components(mod):
     # starting point we assign an appropriate value to all the existing
     # projects here.
     def BuildGen_assign_default_value(m, g, bld_yr):
-        m.BuildGen[g, bld_yr] = m.gen_predetermined_cap[g, bld_yr]
+        m.BuildGen[g, bld_yr] = m.build_gen_predetermined[g, bld_yr]
+
     mod.BuildGen_assign_default_value = BuildAction(
-        mod.PREDETERMINED_GEN_BLD_YRS,
-        rule=BuildGen_assign_default_value)
+        mod.PREDETERMINED_GEN_BLD_YRS, rule=BuildGen_assign_default_value
+    )
 
     # note: in pull request 78, commit e7f870d..., GEN_PERIODS
     # was mistakenly redefined as GENERATION_PROJECTS * PERIODS.
@@ -399,36 +467,42 @@ def define_components(mod):
     # and 'C-Coal_ST' in m.GENS_IN_PERIOD[2020] and 'C-Coal_ST' not in m.GENS_IN_PERIOD[2030]
     mod.GEN_PERIODS = Set(
         dimen=2,
-        initialize=lambda m:
-            [(g, p) for g in m.GENERATION_PROJECTS for p in m.PERIODS_FOR_GEN[g]])
+        initialize=lambda m: [
+            (g, p) for g in m.GENERATION_PROJECTS for p in m.PERIODS_FOR_GEN[g]
+        ],
+    )
 
     mod.GenCapacity = Expression(
-        mod.GENERATION_PROJECTS, mod.PERIODS,
+        mod.GENERATION_PROJECTS,
+        mod.PERIODS,
         rule=lambda m, g, period: sum(
-            m.BuildGen[g, bld_yr]
-            for bld_yr in m.BLD_YRS_FOR_GEN_PERIOD[g, period]))
+            m.BuildGen[g, bld_yr] for bld_yr in m.BLD_YRS_FOR_GEN_PERIOD[g, period]
+        ),
+    )
 
     mod.Max_Build_Potential = Constraint(
-        mod.CAPACITY_LIMITED_GENS, mod.PERIODS,
-        rule=lambda m, g, p: (
-            m.gen_capacity_limit_mw[g] >= m.GenCapacity[g, p]))
+        mod.CAPACITY_LIMITED_GENS,
+        mod.PERIODS,
+        rule=lambda m, g, p: (m.gen_capacity_limit_mw[g] >= m.GenCapacity[g, p]),
+    )
 
     # The following components enforce minimum capacity build-outs.
     # Note that this adds binary variables to the model.
-    mod.gen_min_build_capacity = Param (mod.GENERATION_PROJECTS,
-        within=NonNegativeReals, default=0)
+    mod.gen_min_build_capacity = Param(
+        mod.GENERATION_PROJECTS, within=NonNegativeReals, default=0
+    )
     mod.NEW_GEN_WITH_MIN_BUILD_YEARS = Set(
+        dimen=2,
         initialize=mod.NEW_GEN_BLD_YRS,
-        filter=lambda m, g, p: (
-            m.gen_min_build_capacity[g] > 0))
-    mod.BuildMinGenCap = Var(
-        mod.NEW_GEN_WITH_MIN_BUILD_YEARS,
-        within=Binary)
+        filter=lambda m, g, p: (m.gen_min_build_capacity[g] > 0),
+    )
+    mod.BuildMinGenCap = Var(mod.NEW_GEN_WITH_MIN_BUILD_YEARS, within=Binary)
     mod.Enforce_Min_Build_Lower = Constraint(
         mod.NEW_GEN_WITH_MIN_BUILD_YEARS,
         rule=lambda m, g, p: (
-            m.BuildMinGenCap[g, p] * m.gen_min_build_capacity[g]
-            <= m.BuildGen[g, p]))
+            m.BuildMinGenCap[g, p] * m.gen_min_build_capacity[g] <= m.BuildGen[g, p]
+        ),
+    )
 
     # Define a constant for enforcing binary constraints on project capacity
     # The value of 100 GW should be larger than any expected build size. For
@@ -440,40 +514,48 @@ def define_components(mod):
     mod.Enforce_Min_Build_Upper = Constraint(
         mod.NEW_GEN_WITH_MIN_BUILD_YEARS,
         rule=lambda m, g, p: (
-            m.BuildGen[g, p] <= m.BuildMinGenCap[g, p] *
-                mod._gen_max_cap_for_binary_constraints))
+            m.BuildGen[g, p]
+            <= m.BuildMinGenCap[g, p] * mod._gen_max_cap_for_binary_constraints
+        ),
+    )
 
     # Costs
-    mod.gen_variable_om = Param (mod.GENERATION_PROJECTS, within=NonNegativeReals)
-    mod.gen_connect_cost_per_mw = Param(mod.GENERATION_PROJECTS, within=NonNegativeReals)
-    mod.min_data_check('gen_variable_om', 'gen_connect_cost_per_mw')
+    mod.gen_variable_om = Param(mod.GENERATION_PROJECTS, within=NonNegativeReals)
+    mod.gen_connect_cost_per_mw = Param(
+        mod.GENERATION_PROJECTS, within=NonNegativeReals
+    )
+    mod.min_data_check("gen_variable_om", "gen_connect_cost_per_mw")
 
-    mod.gen_overnight_cost = Param(
-        mod.GEN_BLD_YRS,
-        within=NonNegativeReals)
-    mod.gen_fixed_om = Param(
-        mod.GEN_BLD_YRS,
-        within=NonNegativeReals)
-    mod.min_data_check('gen_overnight_cost', 'gen_fixed_om')
+    mod.gen_overnight_cost = Param(mod.GEN_BLD_YRS, within=NonNegativeReals)
+    mod.gen_fixed_om = Param(mod.GEN_BLD_YRS, within=NonNegativeReals)
+    mod.min_data_check("gen_overnight_cost", "gen_fixed_om")
 
     # Derived annual costs
     mod.gen_capital_cost_annual = Param(
         mod.GEN_BLD_YRS,
+        within=NonNegativeReals,
         initialize=lambda m, g, bld_yr: (
-            (m.gen_overnight_cost[g, bld_yr] +
-                m.gen_connect_cost_per_mw[g]) *
-            crf(m.interest_rate, m.gen_max_age[g])))
+            (m.gen_overnight_cost[g, bld_yr] + m.gen_connect_cost_per_mw[g])
+            * crf(m.interest_rate, m.gen_max_age[g])
+        ),
+    )
 
     mod.GenCapitalCosts = Expression(
-        mod.GENERATION_PROJECTS, mod.PERIODS,
+        mod.GENERATION_PROJECTS,
+        mod.PERIODS,
         rule=lambda m, g, p: sum(
             m.BuildGen[g, bld_yr] * m.gen_capital_cost_annual[g, bld_yr]
-            for bld_yr in m.BLD_YRS_FOR_GEN_PERIOD[g, p]))
+            for bld_yr in m.BLD_YRS_FOR_GEN_PERIOD[g, p]
+        ),
+    )
     mod.GenFixedOMCosts = Expression(
-        mod.GENERATION_PROJECTS, mod.PERIODS,
+        mod.GENERATION_PROJECTS,
+        mod.PERIODS,
         rule=lambda m, g, p: sum(
             m.BuildGen[g, bld_yr] * m.gen_fixed_om[g, bld_yr]
-            for bld_yr in m.BLD_YRS_FOR_GEN_PERIOD[g, p]))
+            for bld_yr in m.BLD_YRS_FOR_GEN_PERIOD[g, p]
+        ),
+    )
     # Summarize costs for the objective function. Units should be total
     # annual future costs in $base_year real dollars. The objective
     # function will convert these to base_year Net Present Value in
@@ -482,8 +564,10 @@ def define_components(mod):
         mod.PERIODS,
         rule=lambda m, p: sum(
             m.GenCapitalCosts[g, p] + m.GenFixedOMCosts[g, p]
-            for g in m.GENERATION_PROJECTS))
-    mod.Cost_Components_Per_Period.append('TotalGenFixedCosts')
+            for g in m.GENERATION_PROJECTS
+        ),
+    )
+    mod.Cost_Components_Per_Period.append("TotalGenFixedCosts")
 
 
 def load_inputs(mod, switch_data, inputs_dir):
@@ -492,7 +576,7 @@ def load_inputs(mod, switch_data, inputs_dir):
     Import data describing project builds. The following files are
     expected in the input directory.
 
-    generation_projects_info.csv has mandatory and optional columns. The
+    gen_info.csv has mandatory and optional columns. The
     operations.gen_dispatch module will also look for additional columns in
     this file. You may drop optional columns entirely or mark blank
     values with a dot '.' for select rows for which the column does not
@@ -510,7 +594,7 @@ def load_inputs(mod, switch_data, inputs_dir):
     optional for simulations where there is no existing capacity:
 
     gen_build_predetermined.csv
-        GENERATION_PROJECT, build_year, gen_predetermined_cap
+        GENERATION_PROJECT, build_year, build_gen_predetermined
 
     The following file is mandatory, because it sets cost parameters for
     both existing and new project buildouts:
@@ -519,64 +603,162 @@ def load_inputs(mod, switch_data, inputs_dir):
         GENERATION_PROJECT, build_year, gen_overnight_cost, gen_fixed_om
 
     """
+
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'generation_projects_info.csv'),
-        auto_select=True,
-        optional_params=['gen_dbid', 'gen_is_baseload', 'gen_scheduled_outage_rate',
-        'gen_forced_outage_rate', 'gen_capacity_limit_mw', 'gen_unit_size',
-        'gen_ccs_energy_load', 'gen_ccs_capture_efficiency',
-        'gen_min_build_capacity', 'gen_is_cogen', 'gen_is_distributed'],
+        filename=os.path.join(inputs_dir, "gen_info.csv"),
+        optional_params=[
+            "gen_dbid",
+            "gen_is_baseload",
+            "gen_scheduled_outage_rate",
+            "gen_forced_outage_rate",
+            "gen_capacity_limit_mw",
+            "gen_unit_size",
+            "gen_ccs_energy_load",
+            "gen_ccs_capture_efficiency",
+            "gen_min_build_capacity",
+            "gen_is_cogen",
+            "gen_is_distributed",
+        ],
         index=mod.GENERATION_PROJECTS,
-        param=(mod.gen_dbid, mod.gen_tech, mod.gen_energy_source,
-               mod.gen_load_zone, mod.gen_max_age, mod.gen_is_variable,
-               mod.gen_is_baseload, mod.gen_scheduled_outage_rate,
-               mod.gen_forced_outage_rate, mod.gen_capacity_limit_mw,
-               mod.gen_unit_size, mod.gen_ccs_energy_load,
-               mod.gen_ccs_capture_efficiency, mod.gen_full_load_heat_rate,
-               mod.gen_variable_om, mod.gen_min_build_capacity,
-               mod.gen_connect_cost_per_mw, mod.gen_is_cogen,
-               mod.gen_is_distributed))
+        param=(
+            mod.gen_dbid,
+            mod.gen_tech,
+            mod.gen_energy_source,
+            mod.gen_load_zone,
+            mod.gen_max_age,
+            mod.gen_is_variable,
+            mod.gen_is_baseload,
+            mod.gen_scheduled_outage_rate,
+            mod.gen_forced_outage_rate,
+            mod.gen_capacity_limit_mw,
+            mod.gen_unit_size,
+            mod.gen_ccs_energy_load,
+            mod.gen_ccs_capture_efficiency,
+            mod.gen_full_load_heat_rate,
+            mod.gen_variable_om,
+            mod.gen_min_build_capacity,
+            mod.gen_connect_cost_per_mw,
+            mod.gen_is_cogen,
+            mod.gen_is_distributed,
+        ),
+    )
     # Construct sets of capacity-limited, ccs-capable and unit-size-specified
     # projects. These sets include projects for which these parameters have
     # a value
-    if 'gen_capacity_limit_mw' in switch_data.data():
-        switch_data.data()['CAPACITY_LIMITED_GENS'] = {
-            None: list(switch_data.data(name='gen_capacity_limit_mw').keys())}
-    if 'gen_unit_size' in switch_data.data():
-        switch_data.data()['DISCRETELY_SIZED_GENS'] = {
-            None: list(switch_data.data(name='gen_unit_size').keys())}
-    if 'gen_ccs_capture_efficiency' in switch_data.data():
-        switch_data.data()['CCS_EQUIPPED_GENS'] = {
-            None: list(switch_data.data(name='gen_ccs_capture_efficiency').keys())}
+    if "gen_capacity_limit_mw" in switch_data.data():
+        switch_data.data()["CAPACITY_LIMITED_GENS"] = {
+            None: list(switch_data.data(name="gen_capacity_limit_mw").keys())
+        }
+    if "gen_unit_size" in switch_data.data():
+        switch_data.data()["DISCRETELY_SIZED_GENS"] = {
+            None: list(switch_data.data(name="gen_unit_size").keys())
+        }
+    if "gen_ccs_capture_efficiency" in switch_data.data():
+        switch_data.data()["CCS_EQUIPPED_GENS"] = {
+            None: list(switch_data.data(name="gen_ccs_capture_efficiency").keys())
+        }
     switch_data.load_aug(
         optional=True,
-        filename=os.path.join(inputs_dir, 'gen_build_predetermined.csv'),
-        auto_select=True,
+        filename=os.path.join(inputs_dir, "gen_build_predetermined.csv"),
         index=mod.PREDETERMINED_GEN_BLD_YRS,
-        param=(mod.gen_predetermined_cap))
+        param=(mod.build_gen_predetermined),
+    )
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'gen_build_costs.csv'),
-        auto_select=True,
+        filename=os.path.join(inputs_dir, "gen_build_costs.csv"),
         index=mod.GEN_BLD_YRS,
-        param=(mod.gen_overnight_cost, mod.gen_fixed_om))
-    # read FUELS_FOR_MULTIFUEL_GEN from gen_multiple_fuels.dat if available
-    multi_fuels_path = os.path.join(inputs_dir, 'gen_multiple_fuels.dat')
-    if os.path.isfile(multi_fuels_path):
-        switch_data.load(filename=multi_fuels_path)
+        param=(mod.gen_overnight_cost, mod.gen_fixed_om),
+    )
+    switch_data.load_aug(
+        optional=True,
+        filename=os.path.join(inputs_dir, "gen_multiple_fuels.csv"),
+        index=mod.MULTI_FUEL_GEN_FUELS,
+        param=tuple(),
+    )
 
 
 def post_solve(m, outdir):
+    # report generator and storage additions in each period and and total
+    # capital outlay for those (up-front capital outlay is not treated as a
+    # direct cost by Switch, but is often interesting to users)
     write_table(
         m,
-        sorted(m.GEN_PERIODS) if m.options.sorted_output else m.GEN_PERIODS,
+        m.GEN_PERIODS,
+        output_file=os.path.join(outdir, "gen_build.csv"),
+        headings=(
+            "GENERATION_PROJECT",
+            "PERIOD",
+            "gen_tech",
+            "gen_load_zone",
+            "gen_energy_source",
+            "BuildGen",
+            "BuildStorageEnergy",
+            "GenCapitalOutlay",
+        ),
+        values=lambda m, g, p: (
+            g,
+            p,
+            m.gen_tech[g],
+            m.gen_load_zone[g],
+            m.gen_energy_source[g],
+            m.BuildGen[g, p] if (g, p) in m.BuildGen else 0.0,
+            (
+                m.BuildStorageEnergy[g, p]
+                if hasattr(m, "BuildStorageEnergy") and (g, p) in m.BuildStorageEnergy
+                else 0.0
+            ),
+            (
+                (m.gen_overnight_cost[g, p] + m.gen_connect_cost_per_mw[g])
+                * m.BuildGen[g, p]
+                if (g, p) in m.BuildGen
+                else 0.0
+            )
+            + (
+                (m.BuildStorageEnergy[g, p] * m.gen_storage_energy_overnight_cost[g, p])
+                if hasattr(m, "BuildStorageEnergy") and (g, p) in m.BuildStorageEnergy
+                else 0.0
+            ),
+        ),
+    )
+
+    # report total generator and storage capacity in place for each generator in
+    # each period. Also show capital and fixed O&M recovery per year in that
+    # period (these are the costs Switch seeks to minimize)
+    write_table(
+        m,
+        m.GENERATION_PROJECTS,
+        m.PERIODS,
         output_file=os.path.join(outdir, "gen_cap.csv"),
         headings=(
-            "GENERATION_PROJECT", "PERIOD",
-            "gen_tech", "gen_load_zone", "gen_energy_source",
-            "GenCapacity", "GenCapitalCosts", "GenFixedOMCosts"),
-        # Indexes are provided as a tuple, so put (g,p) in parentheses to
-        # access the two components of the index individually.
+            "GENERATION_PROJECT",
+            "PERIOD",
+            "gen_tech",
+            "gen_load_zone",
+            "gen_energy_source",
+            "GenCapacity",
+            "GenStorageCapacity",
+            "GenCapitalRecovery",
+            "GenFixedOMCosts",
+        ),
         values=lambda m, g, p: (
-            g, p,
-            m.gen_tech[g], m.gen_load_zone[g], m.gen_energy_source[g],
-            m.GenCapacity[g, p], m.GenCapitalCosts[g, p], m.GenFixedOMCosts[g, p]))
+            g,
+            p,
+            m.gen_tech[g],
+            m.gen_load_zone[g],
+            m.gen_energy_source[g],
+            m.GenCapacity[g, p],
+            (
+                m.StorageEnergyCapacity[g, p]
+                if hasattr(m, "StorageEnergyCapacity")
+                and (g, p) in m.StorageEnergyCapacity
+                else 0.0
+            ),
+            m.GenCapitalCosts[g, p]
+            + (
+                m.StorageEnergyCapitalCost[g, p]
+                if hasattr(m, "StorageEnergyCapitalCost")
+                and (g, p) in m.StorageEnergyCapitalCost
+                else 0.0
+            ),
+            m.GenFixedOMCosts[g, p],
+        ),
+    )
