@@ -10,12 +10,12 @@ def define_components(m):
     """
 
     # TODO: put these in a data file and share them between rps.py and no_renewables.py
-    renewable_energy_sources = ['WND']
+    renewable_energy_sources = ["WND"]
 
     def No_Wind_rule(m, g, bld_yr):
         if m.g_energy_source[m.gen_tech[g]] in renewable_energy_sources:
             return m.BuildGen[g, bld_yr] == 0
         else:
             return Constraint.Skip
-    m.No_Wind = Constraint(m.NEW_GEN_BLD_YRS, rule=No_Wind_rule)
 
+    m.No_Wind = Constraint(m.NEW_GEN_BLD_YRS, rule=No_Wind_rule)
