@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2019 The Switch Authors. All rights reserved.
+# Copyright (c) 2015-2022 The Switch Authors. All rights reserved.
 # Licensed under the Apache License, Version 2.0, which is in the LICENSE file.
 
 """
@@ -227,24 +227,28 @@ def define_components(mod):
 
     """
 
-    mod.PERIODS = Set(ordered=True)
+    mod.PERIODS = Set(dimen=1, ordered=True)
     mod.period_start = Param(mod.PERIODS, within=NonNegativeReals)
     mod.period_end = Param(mod.PERIODS, within=NonNegativeReals)
-    mod.min_data_check('PERIODS', 'period_start', 'period_end')
+    mod.min_data_check("PERIODS", "period_start", "period_end")
 
-    mod.TIMESERIES = Set(ordered=True)
+    mod.TIMESERIES = Set(ordered=True, dimen=1)
     mod.ts_period = Param(mod.TIMESERIES, within=mod.PERIODS)
-    mod.ts_duration_of_tp = Param(mod.TIMESERIES, within=PositiveReals)
-    mod.ts_num_tps = Param(mod.TIMESERIES, within=PositiveIntegers)
-    mod.ts_scale_to_period = Param(mod.TIMESERIES, within=PositiveReals)
+    mod.ts_duration_of_tp = Param(mod.TIMESERIES, within=NonNegativeReals)
+    mod.ts_num_tps = Param(mod.TIMESERIES, within=NonNegativeIntegers)
+    mod.ts_scale_to_period = Param(mod.TIMESERIES, within=NonNegativeReals)
     mod.min_data_check(
-        'TIMESERIES', 'ts_period', 'ts_duration_of_tp', 'ts_num_tps',
-        'ts_scale_to_period')
+        "TIMESERIES",
+        "ts_period",
+        "ts_duration_of_tp",
+        "ts_num_tps",
+        "ts_scale_to_period",
+    )
 
-    mod.TIMEPOINTS = Set(ordered=True)
+    mod.TIMEPOINTS = Set(ordered=True, dimen=1)
     mod.tp_ts = Param(mod.TIMEPOINTS, within=mod.TIMESERIES)
-    mod.min_data_check('TIMEPOINTS', 'tp_ts')
-    mod.tp_timestamp = Param(mod.TIMEPOINTS, default=lambda m, t: t)
+    mod.min_data_check("TIMEPOINTS", "tp_ts")
+    mod.tp_timestamp = Param(mod.TIMEPOINTS, default=lambda m, t: t, within=Any)
 
     # Derived sets and parameters
     # note: the first five are calculated early so they
@@ -252,34 +256,43 @@ def define_components(mod):
 
     mod.tp_duration_hrs = Param(
         mod.TIMEPOINTS,
-        initialize=lambda m, t: m.ts_duration_of_tp[m.tp_ts[t]])
+        within=NonNegativeReals,
+        initialize=lambda m, t: m.ts_duration_of_tp[m.tp_ts[t]],
+    )
     mod.tp_weight = Param(
         mod.TIMEPOINTS,
-        within=PositiveReals,
+        within=NonNegativeReals,
         initialize=lambda m, t: (
-            m.tp_duration_hrs[t] * m.ts_scale_to_period[m.tp_ts[t]]))
+            m.tp_duration_hrs[t] * m.ts_scale_to_period[m.tp_ts[t]]
+        ),
+    )
+    # TODO: build this in one pass, not multiple scans
     mod.TPS_IN_TS = Set(
         mod.TIMESERIES,
+        dimen=1,
         ordered=True,
         within=mod.TIMEPOINTS,
-        initialize=lambda m, ts: [
-            t for t in m.TIMEPOINTS if m.tp_ts[t] == ts])
+        initialize=lambda m, ts: [t for t in m.TIMEPOINTS if m.tp_ts[t] == ts],
+    )
     mod.tp_period = Param(
         mod.TIMEPOINTS,
         within=mod.PERIODS,
-        initialize=lambda m, t: m.ts_period[m.tp_ts[t]])
+        initialize=lambda m, t: m.ts_period[m.tp_ts[t]],
+    )
     mod.TS_IN_PERIOD = Set(
         mod.PERIODS,
+        dimen=1,
         ordered=True,
         within=mod.TIMESERIES,
-        initialize=lambda m, p: [
-            ts for ts in m.TIMESERIES if m.ts_period[ts] == p])
+        initialize=lambda m, p: [ts for ts in m.TIMESERIES if m.ts_period[ts] == p],
+    )
     mod.TPS_IN_PERIOD = Set(
         mod.PERIODS,
+        dimen=1,
         ordered=True,
         within=mod.TIMEPOINTS,
-        initialize=lambda m, p: [
-            t for t in m.TIMEPOINTS if m.tp_period[t] == p])
+        initialize=lambda m, p: [t for t in m.TIMEPOINTS if m.tp_period[t] == p],
+    )
 
     # Decide whether period_end values have been given as exact points in time
     # (e.g., 2020.0 means 2020-01-01 00:00:00), or as a label for a full
@@ -288,45 +301,67 @@ def define_components(mod):
     # NOTE: we can't just check whether period_end[p] + 1 = period_start[p+1],
     # because that is undefined for single-period models.
     def add_one_to_period_end_rule(m):
-        hours_in_period = {p: sum(m.tp_weight[t] for t in m.TPS_IN_PERIOD[p]) for p in m.PERIODS}
+        hours_in_period = {
+            p: sum(m.tp_weight[t] for t in m.TPS_IN_PERIOD[p]) for p in m.PERIODS
+        }
         err_plain = sum(
             (m.period_end[p] - m.period_start[p]) * hours_per_year - hours_in_period[p]
-                for p in m.PERIODS)
+            for p in m.PERIODS
+        )
         err_add_one = sum(
-            (m.period_end[p] + 1 - m.period_start[p]) * hours_per_year - hours_in_period[p]
-                for p in m.PERIODS)
-        add_one = (abs(err_add_one) < abs(err_plain))
+            (m.period_end[p] + 1 - m.period_start[p]) * hours_per_year
+            - hours_in_period[p]
+            for p in m.PERIODS
+        )
+        add_one = abs(err_add_one) < abs(err_plain)
         # print "add_one: {}".format(add_one)
         return add_one
-    mod.add_one_to_period_end = Param(within=Boolean, initialize=add_one_to_period_end_rule)
+
+    mod.add_one_to_period_end = Param(
+        within=Boolean, initialize=add_one_to_period_end_rule
+    )
 
     mod.period_length_years = Param(
         mod.PERIODS,
-        initialize=lambda m, p: m.period_end[p] - m.period_start[p] + (1 if m.add_one_to_period_end else 0))
+        within=NonNegativeReals,
+        initialize=lambda m, p: m.period_end[p]
+        - m.period_start[p]
+        + (1 if m.add_one_to_period_end else 0),
+    )
     mod.period_length_hours = Param(
         mod.PERIODS,
-        initialize=lambda m, p: m.period_length_years[p] * hours_per_year)
+        within=NonNegativeReals,
+        initialize=lambda m, p: m.period_length_years[p] * hours_per_year,
+    )
 
     mod.CURRENT_AND_PRIOR_PERIODS_FOR_PERIOD = Set(
-        mod.PERIODS, ordered=True,
-        initialize=lambda m, p:
-            [p2 for p2 in m.PERIODS if m.PERIODS.ord(p2) <= m.PERIODS.ord(p)]
+        mod.PERIODS,
+        dimen=1,
+        ordered=True,
+        initialize=lambda m, p: [
+            p2 for p2 in m.PERIODS if m.PERIODS.ord(p2) <= m.PERIODS.ord(p)
+        ],
     )
 
     mod.ts_scale_to_year = Param(
         mod.TIMESERIES,
+        within=NonNegativeReals,
         initialize=lambda m, ts: (
-            m.ts_scale_to_period[ts] / m.period_length_years[m.ts_period[ts]]))
+            m.ts_scale_to_period[ts] / m.period_length_years[m.ts_period[ts]]
+        ),
+    )
     mod.ts_duration_hrs = Param(
         mod.TIMESERIES,
-        initialize=lambda m, ts: (
-            m.ts_num_tps[ts] * m.ts_duration_of_tp[ts]))
+        within=NonNegativeReals,
+        initialize=lambda m, ts: (m.ts_num_tps[ts] * m.ts_duration_of_tp[ts]),
+    )
 
     mod.tp_weight_in_year = Param(
         mod.TIMEPOINTS,
-        within=PositiveReals,
-        initialize=lambda m, t: (
-            m.tp_weight[t] / m.period_length_years[m.tp_period[t]]))
+        within=NonNegativeReals,
+        initialize=lambda m, t: m.tp_weight[t] / m.period_length_years[m.tp_period[t]],
+        doc="This weight scales a timepoint to an annual average.",
+    )
     # Identify previous step for each timepoint, for use in tracking
     # unit commitment or storage. We use circular indexing (.prevw() method)
     # for the timepoints within a timeseries to give consistency between the
@@ -335,24 +370,30 @@ def define_components(mod):
     mod.tp_previous = Param(
         mod.TIMEPOINTS,
         within=mod.TIMEPOINTS,
-        initialize=lambda m, t: m.TPS_IN_TS[m.tp_ts[t]].prevw(t))
+        initialize=lambda m, t: m.TPS_IN_TS[m.tp_ts[t]].prevw(t),
+    )
 
     def validate_time_weights_rule(m, p):
         hours_in_period = sum(m.tp_weight[t] for t in m.TPS_IN_PERIOD[p])
         tol = 0.01
-        if(hours_in_period > (1 + tol) * m.period_length_hours[p] or
-           hours_in_period < (1 - tol) * m.period_length_hours[p]):
-            print(("validate_time_weights_rule failed for period " +
-                   "'{period:.0f}'. Expected {period_h:0.2f}, based on " +
-                   "length in years, but the sum of timepoint weights " +
-                   "is {ds_h:0.2f}.\n"
-                   ).format(period=p, period_h=m.period_length_hours[p],
-                            ds_h=hours_in_period))
+        if (
+            hours_in_period > (1 + tol) * m.period_length_hours[p]
+            or hours_in_period < (1 - tol) * m.period_length_hours[p]
+        ):
+            print(
+                (
+                    "validate_time_weights_rule failed for period "
+                    + "'{period:.0f}'. Expected {period_h:0.2f}, based on "
+                    + "length in years, but the sum of timepoint weights "
+                    + "is {ds_h:0.2f}.\n"
+                ).format(
+                    period=p, period_h=m.period_length_hours[p], ds_h=hours_in_period
+                )
+            )
             return 0
         return 1
-    mod.validate_time_weights = BuildCheck(
-        mod.PERIODS,
-        rule=validate_time_weights_rule)
+
+    mod.validate_time_weights = BuildCheck(mod.PERIODS, rule=validate_time_weights_rule)
 
     def validate_period_lengths_rule(m, p):
         tol = 0.01
@@ -360,16 +401,19 @@ def define_components(mod):
             p_end = m.period_start[p] + m.period_length_years[p]
             p_next = m.period_start[m.PERIODS.next(p)]
             if abs(p_next - p_end) > tol:
-                print((
-                    "validate_period_lengths_rule failed for period"
-                    + "'{p:.0f}'. Period ends at {p_end}, but next period"
-                    + "begins at {p_next}."
-                ).format(p=p, p_end=p_end, p_next=p_next))
+                print(
+                    (
+                        "validate_period_lengths_rule failed for period"
+                        + "'{p:.0f}'. Period ends at {p_end}, but next period"
+                        + "begins at {p_next}."
+                    ).format(p=p, p_end=p_end, p_next=p_next)
+                )
                 return False
         return True
+
     mod.validate_period_lengths = BuildCheck(
-        mod.PERIODS,
-        rule=validate_period_lengths_rule)
+        mod.PERIODS, rule=validate_period_lengths_rule
+    )
 
 
 def load_inputs(mod, switch_data, inputs_dir):
@@ -399,19 +443,23 @@ def load_inputs(mod, switch_data, inputs_dir):
     # names, be indifferent to column order, and throw an error message if
     # some columns are not found.
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'periods.csv'),
-        select=('INVESTMENT_PERIOD', 'period_start', 'period_end'),
+        filename=os.path.join(inputs_dir, "periods.csv"),
         index=mod.PERIODS,
-        param=(mod.period_start, mod.period_end))
+        param=(mod.period_start, mod.period_end),
+    )
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'timeseries.csv'),
-        select=('TIMESERIES', 'ts_period', 'ts_duration_of_tp',
-                'ts_num_tps', 'ts_scale_to_period'),
+        filename=os.path.join(inputs_dir, "timeseries.csv"),
         index=mod.TIMESERIES,
-        param=(mod.ts_period, mod.ts_duration_of_tp,
-               mod.ts_num_tps, mod.ts_scale_to_period))
+        param=(
+            mod.ts_period,
+            mod.ts_duration_of_tp,
+            mod.ts_num_tps,
+            mod.ts_scale_to_period,
+        ),
+    )
     switch_data.load_aug(
-        filename=os.path.join(inputs_dir, 'timepoints.csv'),
-        select=('timepoint_id', 'timestamp', 'timeseries'),
+        filename=os.path.join(inputs_dir, "timepoints.csv"),
+        select=("timepoint_id", "timestamp", "timeseries"),
         index=mod.TIMEPOINTS,
-        param=(mod.tp_timestamp, mod.tp_ts))
+        param=(mod.tp_timestamp, mod.tp_ts),
+    )
